@@ -17,7 +17,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 __author__ = 'Chris Mitchell'
 
-import sys, copy, struct, base64, gzip
+import sys, copy, struct, base64, gzip, operator
 from os import path
 import pythomics.templates as templates
 from pythomics.proteomics.structures import PeptideObject, ScanObject, Chromatogram
@@ -48,9 +48,37 @@ class GenericProteomicIterator(object):
     def getBaseTrace(self):
         pass
 
+    def getPeptide(self, peptide=None):
+        pass
+
+class GuessIterator(templates.GenericIterator):
+    def __init__(self, *args, **kwargs):
+        super(GuessIterator, self).__init__(*args, **kwargs)
+        filename = self.filename.name.lower()
+        if '.mzml' in filename:
+            self.parser = MZMLIterator
+        elif 'pepxml' in filename:
+            self.parser = PepXMLIterator
+        elif '.xml' in filename:
+            self.parser = XTandemXMLIterator
+        elif '.mgf' in filename:
+            self.parser = MGFIterator
+        elif '.dat' in filename:
+            self.parser = MascotDATIterator
+        elif '.msf' in filename or '.pdresult' in filename:
+            self.parser = ThermoMSFIterator
+        self.parser = self.parser(*args, **kwargs)
+        private = {'__iter__',}
+        for i in dir(self.parser):
+            if i.startswith('__') and i not in private:
+                continue
+            setattr(self, i, getattr(self.parser, i))
+
+    def next(self):
+        return self.parser.next()
 
 class MZMLIterator(templates.GenericIterator, GenericProteomicIterator):
-    def __init__(self, filename):
+    def __init__(self, filename, full=False, store=True, ms_filter=False, start=0):
         """An iterator over the mzML file format.
          I just need a parser right now so the specs might not be exactly correct
 
@@ -59,6 +87,10 @@ class MZMLIterator(templates.GenericIterator, GenericProteomicIterator):
         """
         super(MZMLIterator, self).__init__(filename)
         self.scans = {}
+        self.store = store
+        self.full = full
+        self.ms_filter = ms_filter
+        self.start = start
         if isinstance(self.filename, gzip.GzipFile):
             self.gzip = True
         else:
@@ -96,9 +128,9 @@ class MZMLIterator(templates.GenericIterator, GenericProteomicIterator):
         else:
             array = base64.b64decode(array)
         if '64-bit float' in params:
-            array = [struct.unpack('d', array[i:i+8])[0] for i in xrange(0,len(array),8)]
+            array = [struct.unpack('d', array[i:i+8])[0] for i in xrange(0, len(array), 8)]
         else:
-            array = [struct.unpack('f', array[i:i+4])[0] for i in xrange(0,len(array),4)]
+            array = [struct.unpack('f', array[i:i+4])[0] for i in xrange(0, len(array), 4)]
         return array
 
     def parselxml(self, spectra, full=False, namespace='{http://psi.hupo.org/ms/mzml}'):
@@ -108,14 +140,7 @@ class MZMLIterator(templates.GenericIterator, GenericProteomicIterator):
             return None
         elif spectra.tag == '{0}spectrum'.format(namespace):
             scanObj = ScanObject()
-            if full:
-                mzmls, intensities = spectra.findall('{0}binaryDataArrayList/'.format(namespace))
-                mzml_params = dict([(i.get('name'), i.get('value')) for i in mzmls.findall('{0}cvParam'.format(namespace))])
-                mzmls = self.unpack_array(mzmls.find('{0}binary'.format(namespace)).text, mzml_params, namespace=namespace)
-                intensity_params = dict([(i.get('name'), i.get('value')) for i in intensities.findall('{0}cvParam'.format(namespace))])
-                intensities = self.unpack_array(intensities.find('{0}binary'.format(namespace)).text, intensity_params, namespace=namespace)
-                scanObj.scans = [(i,j) for i,j in zip(mzmls, intensities)]
-            spectra_info = dict(zip(spectra.keys(),spectra.values()))
+            # spectra_info = dict(zip(spectra.keys(),spectra.values()))
             spectra_params = dict([(i.get('name'), i.get('value')) for i in spectra.findall('{0}cvParam'.format(namespace))])
             scan_info = dict([(i.get('name'), i.get('value')) for i in spectra.findall('{0}scanList/{0}scan/{0}cvParam'.format(namespace))])
             precursor_info = dict([(i.get('name'), i.get('value')) for i in spectra.findall('{0}precursorList/{0}precursor/{0}selectedIonList/{0}selectedIon/{0}cvParam'.format(namespace))])
@@ -123,17 +148,30 @@ class MZMLIterator(templates.GenericIterator, GenericProteomicIterator):
             # print 'spectra params', spectra_params
             # print 'our scan info', scan_info
             # print 'our precursor info', precursor_info
-            ms_level = spectra_params.get('ms level', 0)
+            ms_level = int(spectra_params.get('ms level', 0))
             charge = precursor_info.get('charge state', 0)
             precursor_ion = precursor_info.get('selected ion m/z', 0)
-            precursor_intensity = precursor_info.get('peak intensity', 0)
+            # precursor_intensity = precursor_info.get('peak intensity', 0)
             rt = scan_info.get('scan start time', 0)
-            rt_length = scan_info.get('ion injection time', 0)
-            scanObj.ms_level = int(ms_level)
+            # rt_length = scan_info.get('ion injection time', 0)
+            scanObj.ms_level = ms_level
             scanObj.mass = float(precursor_ion)
             scanObj.charge = charge
-            scanObj.title = dict([i.split('=') for i in spectra.get('id').split(' ')]).get('scan', 'No Title')
+            title = dict([i.split('=') for i in spectra.get('id').split(' ')]).get('scan', 'No Title')
+            scanObj.title = title
             scanObj.rt = float(rt)
+            try:
+                title = int(title)
+            except:
+                pass
+            if title > self.start and (not self.ms_filter or ms_level==self.ms_filter) and full:
+                mzmls, intensities = spectra.findall('{0}binaryDataArrayList/'.format(namespace))
+                mzml_params = dict([(i.get('name'), i.get('value')) for i in mzmls.findall('{0}cvParam'.format(namespace))])
+                mzmls = self.unpack_array(mzmls.find('{0}binary'.format(namespace)).text, mzml_params, namespace=namespace)
+                intensity_params = dict([(i.get('name'), i.get('value')) for i in intensities.findall('{0}cvParam'.format(namespace))])
+                intensities = self.unpack_array(intensities.find('{0}binary'.format(namespace)).text, intensity_params, namespace=namespace)
+                scanObj.scans = [(i,j) for i,j in zip(mzmls, intensities)]
+            spectra.clear()
             return scanObj
         elif spectra.tag == '{0}chromatogramList'.format(namespace):
             spectra_params = dict([(i.get('name'), i.get('value')) for i in spectra.findall('{0}chromatogram/{0}cvParam'.format(namespace))])
@@ -147,10 +185,11 @@ class MZMLIterator(templates.GenericIterator, GenericProteomicIterator):
             chromObj.times = time_array
             chromObj.intensities = intensities
             self.chromatogram = chromObj
+            spectra.clear()
             return None
         # elif spect:
         #     raise StopIteration
-    
+
     def next(self):
         if self.spectra:
             spectra = self.spectra.next()
@@ -158,11 +197,11 @@ class MZMLIterator(templates.GenericIterator, GenericProteomicIterator):
             raise StopIteration
         if self.gzip:
             scan = self.parselxml(spectra, full=True)
-            if isinstance(scan, ScanObject):
+            if self.store and isinstance(scan, ScanObject):
                 self.scans[scan.title] = scan
             return scan
         else:
-            return self.parselxml(spectra[1])
+            return self.parselxml(spectra[1], full=self.full)
 
     def getChromatogram(self):
         return self.chromatogram
@@ -337,6 +376,110 @@ class PepXMLIterator(templates.GenericIterator, GenericProteomicIterator):
             sys.stderr.write('pepxml cannto find %s\n'%id)
             return None
 
+class mzDataIterator(templates.GenericIterator, GenericProteomicIterator):
+    #TODO: Random access, fetch scan by id
+    def __init__(self, filename, **kwargs):
+        """An iterator over the mzData file format.
+
+        mzData doesn't have random access built in, for now this is more of a reader
+        class since we read in all the peptides then match them up while we iterate over
+        spectra. This is also clunky and not optimized because I rarely use this format, if ever.
+
+        The returned items are PeptideObjects
+
+        """
+        super(mzDataIterator, self).__init__(filename)
+        self.peptides = {}
+        self.store = kwargs.get('store', False)
+        try:
+            # find our peptides and process them first
+            peps = etree.iterparse(self.filename, tag=('PeptideItem',))
+            for tag, peptideXML in peps:
+                pepObj = PeptideObject()
+                peptide = peptideXML.find('Sequence').text
+                spectrum = peptideXML.find('SpectrumReference').text
+                pepObj.rawId = spectrum
+                pepObj.title = spectrum
+                pepObj.id = spectrum
+                pepObj.peptide = peptide
+                additional = peptideXML.find('additional')
+                userParams = dict([(j.get('name'),j.get('value')) for j in additional.findall('userParam')])
+                cvParam = dict([(j.get('name'),j.get('value')) for j in additional.findall('cvParam')])
+                pepObj.charge = cvParam.get('charge state', None)
+                pepObj.mass = userParams.get('Experimental Mass')
+                modifications = peptideXML.iterfind('ModificationItem')
+                for modification in modifications:
+                    position = int(modification.find('ModLocation').text)-1
+                    mod_aa = peptide[position]
+                    mass = modification.find('ModMonoDelta').text
+                    additional = modification.find('additional')
+                    userParams = dict([(j.get('name'),j.get('value')) for j in additional.findall('userParam')])
+                    mod_name = userParams.get('Name')
+                    pepObj.addModification(mod_aa, position, mod_name, mass)
+                self.peptides[spectrum] = pepObj
+            self.filename.seek(0)
+            dom1 = etree.iterparse(self.filename, tag=('spectrum',))
+            self.lxml = True
+        except NameError:
+            self.lxml = False
+            sys.stderr.write('mzData parsing unavailable: lxml is required to parse mzData files\n')
+            return
+        if self.lxml:
+            self.spectra = dom1
+            self.scans = {}
+            self.ra = {}
+        else:
+            #this isn't implemented
+            self.nest = 0
+        self.ptm_regex = re.compile(r'PTMProphet\_[A-Za-z]+([0-9\.\-]+)')
+        self.db = None
+
+    def __iter__(self):
+        return self
+
+    def unpack_array(self, array, params, namespace='{http://psi.hupo.org/ms/mzml}'):
+        if 'zlib compression' in params:
+            import zlib
+            array = zlib.decompress(base64.b64decode(array))
+        else:
+            array = base64.b64decode(array)
+        if params.get('precision', False) == '64':
+            array = [struct.unpack('d', array[i:i+8])[0] for i in xrange(0, len(array), 8)]
+        else:
+            array = [struct.unpack('f', array[i:i+4])[0] for i in xrange(0, len(array), 4)]
+        return array
+
+    def parselxml(self, spectra, full=False, namespace=''):
+        scanId = int(spectra.get('id'))
+        scanObj = self.peptides.pop(scanId, ScanObject)
+        precursorInfo = spectra.iterfind('spectrumDesc/precursorList/precursor/ionSelection')
+        for precursor in precursorInfo:
+            cvParams = dict([(j.get('name'),j.get('value')) for j in precursor.findall('cvParam')])
+            scanObj.rt = cvParams.get('retention time')
+        # get our search result
+        mzData = spectra.find('mzArrayBinary/data')
+        intData = spectra.find('intenArrayBinary/data')
+        mzData = self.unpack_array(mzData.text, dict(mzData.items()), namespace='')
+        intData = self.unpack_array(intData.text, dict(intData.items()), namespace='')
+        scanObj.scans = sorted([(i, j) for i,j in zip(mzData, intData)], key=operator.itemgetter(0))
+        if self.store:
+            self.scans[scanId] = scanObj
+        return scanObj
+
+    def next(self):
+        if self.spectra:
+            spectra = self.spectra.next()
+        else:
+            raise StopIteration
+        return self.parselxml(spectra[1])
+
+    def getScan(self, id):
+        try:
+            return self.scans[id]
+        except IndexError:
+            sys.stderr.write('mzData cannto find %s\n'%id)
+            return None
+
 class XTandemXMLIterator(templates.GenericIterator, GenericProteomicIterator):
     """
     Parser for X!Tandem XML Files.
@@ -857,7 +1000,10 @@ class MascotDATIterator(templates.GenericIterator, GenericProteomicIterator):
         return None
 
 class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
-    def __init__(self, filename, clvl=3,srank=1):
+    def __init__(self, filename, clvl=3, srank=1, full=False, store=False):
+        self.clvl = clvl
+        self.full = full
+        self.srank = srank
         if isinstance(filename,(str,unicode)):
             self.f = open(filename, 'rb')
         else:
@@ -865,20 +1011,38 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
         self.conn = sqlite3.connect(filename, check_same_thread=False)
         #self.conn.row_factory = sqlite3.Row
         self.cur = self.conn.cursor()
-        sql = 'select * from fileinfos'
-        self.cur.execute(sql)
         self.fileMap = {}
         self.sFileMap = {}
-        for i in self.cur.fetchall():
-            self.fileMap[str(i[0])]=str(i[1])
-            self.sFileMap[i[0]]=lastSplit.search(i[1]).group(1)
+        try:
+            sql = 'select * from fileinfos'
+            self.cur.execute(sql)
+            self.version = 1
+            for i in self.cur.fetchall():
+                self.fileMap[str(i[0])]=str(i[1])
+                self.sFileMap[i[0]]=lastSplit.search(i[1]).group(1)
+        except sqlite3.OperationalError:
+            sql = 'select * from analysisdefinition'
+            self.cur.execute(sql)
+            self.version = 2
+            for i in self.cur.fetchall():
+                xml = i[1]
+                self.root = etree.fromstring(xml)
+            sql = 'select FileID, FileName from workflowinputfiles'
+            self.cur.execute(sql)
+            for i in self.cur.fetchall():
+                self.fileMap[str(i[0])]=str(i[1])
+                self.sFileMap[i[0]]=lastSplit.search(i[1]).group(1)
+
+
         #modification table
         sql = 'select a.AminoAcidModificationID,a.ModificationName, a.DeltaMass from aminoacidmodifications a'
         self.cur.execute(sql)
         self.modTable = {}
         self.scans = []
+        self.scanTable = {}
         for i in self.cur.fetchall():
             self.modTable[i[0]] = (i[1],i[2])
+        self.aaTable = dict([(i[0],i[1]) for i in self.conn.execute("select aa.AminoAcidID, aa.OneLetterCode from aminoacids aa").fetchall()])
         #We fetch all modifications here for temporary storage because it is VERY expensive to query peptide by peptide (3 seconds per 100 on my 500 MB test file, with 300,000 scans that's horrid)
         sql = 'select pam.PeptideID, GROUP_CONCAT(pam.AminoAcidModificationID), GROUP_CONCAT(pam.Position) from peptidesaminoacidmodifications pam GROUP BY pam.PeptideID'
         self.cur.execute(sql)
@@ -889,7 +1053,10 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
             sql = 'select COUNT(distinct p.SpectrumID) from peptides p where p.PeptideID IS NOT NULL and p.ConfidenceLevel >= %d and p.SearchEngineRank <= %d'%(clvl,srank)
             self.nrows = self.conn.execute(sql).fetchone()[0]
             #sql = 'select sp.spectrum,p.ConfidenceLevel,p.SearchEngineRank,p.Sequence,p.PeptideID,pp.ProteinID,p.SpectrumID from spectra sp left join peptides p on (p.SpectrumID=sp.UniqueSpectrumID) left join peptidesproteins pp on (p.PeptideID=pp.PeptideID) where p.PeptideID IS NOT NULL'
-            sql = 'select GROUP_CONCAT(p.ConfidenceLevel),GROUP_CONCAT(p.SearchEngineRank),GROUP_CONCAT(p.Sequence),GROUP_CONCAT(p.PeptideID), GROUP_CONCAT(pp.ProteinID), p.SpectrumID, sh.Charge, sh.RetentionTime, sh.FirstScan, sh.LastScan, mp.FileID from peptides p join peptidesproteins pp on (p.PeptideID=pp.PeptideID) left join spectrumheaders sh on (sh.SpectrumID=p.SpectrumID) left join masspeaks mp on (sh.MassPeakID=mp.MassPeakID) where p.PeptideID IS NOT NULL and p.ConfidenceLevel >= %d and p.SearchEngineRank <= %d GROUP BY p.SpectrumID'%(clvl,srank)
+            if full:
+                sql = 'select GROUP_CONCAT(p.ConfidenceLevel),GROUP_CONCAT(p.SearchEngineRank),GROUP_CONCAT(p.Sequence),GROUP_CONCAT(p.PeptideID), GROUP_CONCAT(pp.ProteinID), p.SpectrumID, sh.Charge, sh.RetentionTime, sh.FirstScan, sh.LastScan, mp.FileID, sp.Spectrum from peptides p join peptidesproteins pp on (p.PeptideID=pp.PeptideID) left join spectrumheaders sh on (sh.SpectrumID=p.SpectrumID) left join spectra sp on (sp.UniqueSpectrumID=sh.UniqueSpectrumID) left join masspeaks mp on (sh.MassPeakID=mp.MassPeakID) where p.PeptideID IS NOT NULL and p.ConfidenceLevel >= %d and p.SearchEngineRank <= %d GROUP BY p.SpectrumID'%(clvl,srank)
+            else:
+                sql = 'select GROUP_CONCAT(p.ConfidenceLevel),GROUP_CONCAT(p.SearchEngineRank),GROUP_CONCAT(p.Sequence),GROUP_CONCAT(p.PeptideID), GROUP_CONCAT(pp.ProteinID), p.SpectrumID, sh.Charge, sh.RetentionTime, sh.FirstScan, sh.LastScan, mp.FileID from peptides p join peptidesproteins pp on (p.PeptideID=pp.PeptideID) left join spectrumheaders sh on (sh.SpectrumID=p.SpectrumID) left join masspeaks mp on (sh.MassPeakID=mp.MassPeakID) where p.PeptideID IS NOT NULL and p.ConfidenceLevel >= %d and p.SearchEngineRank <= %d GROUP BY p.SpectrumID'%(clvl,srank)
             self.cur.execute(sql)
         except sqlite3.OperationalError:
             sql = 'select COUNT(distinct p.SpectrumID) from peptides p where p.PeptideID IS NOT NULL and p.ConfidenceLevel >= %d'%clvl
@@ -900,6 +1067,27 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
         self.index = 0
         self.master_scans = {}
 
+    def getSILACLabels(self):
+        masses = {}
+        if self.version == 1:
+            sql = "select pnp.ParameterValue from processingnodeparameters pnp where pnp.ValueDisplayString like 'Label%' and pnp.ParameterValue like '%#%'"
+            entries = self.conn.execute(sql).fetchall()
+            for i in entries:
+                entry = i[0].split('#')
+                mod = int(entry.pop())
+                mod_mass = self.modTable[mod][1]
+                for aa_id in entry:
+                    aa = self.aaTable[int(aa_id)]
+                    masses[aa] = mod_mass
+        elif self.version == 2:
+            import HTMLParser
+            html_parser = HTMLParser.HTMLParser()
+            silac = etree.fromstring([i for i in self.root.iterdescendants('QuantitationMethod')][0].text.encode('utf-16'))
+            for mod_info in [etree.fromstring(html_parser.unescape(i.text)) for i in silac.iterdescendants('Parameter') if 'Modification Version' in i.text]:
+                for aa in mod_info.get('AminoAcids').split(','):
+                    masses[aa] = float(mod_info.get('DeltaMass'))
+        return masses
+
     def loadChromatogram(self):
         sql = 'select * from chromatograms'
         self.cur.execute(sql)
@@ -943,60 +1131,38 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
         i = self.cur.fetchone()
         if not i:
             return None
-        return self.parseFullScan(i)
+        scan = self.parseFullScan(i)
+        scan.spectrumId = specId
+        return scan
 
-    def __iter__(self):
-        return self
-
-    def parseScan(self, i):
-#sql = 'select GROUP_CONCAT(p.ConfidenceLevel),GROUP_CONCAT(p.SearchEngineRank),GROUP_CONCAT(p.Sequence),GROUP_CONCAT(p.PeptideID), GROUP_CONCAT(pp.ProteinID), p.SpectrumID, sh.Charge, sh.RetentionTime, sh.FirstScan, sh.LastScan, mp.FileID from peptides p join peptidesproteins pp on (p.PeptideID=pp.PeptideID) left join spectrumheaders sh on (sh.SpectrumID=p.SpectrumID) left join masspeaks mp on (sh.MassPeakID=mp.MassPeakID) where p.PeptideID IS NOT NULL and p.ConfidenceLevel = 1 and p.SearchEngineRank = 1 GROUP BY p.SpectrumID'
-        objs = []
-        self.index+=1
-        added = set([])#for some reason redundant scans appear
-        for confidence, searchRank, sequence, pepId in zip(i[0].split(','),i[1].split(','),i[2].split(','),i[3].split(',')):
-            if (sequence,pepId) in added:
-                continue
-            else:
-                added.add((sequence,pepId))
-            scanObj = PeptideObject()
+    def getScans(self, modifications=True, fdr=True):
+        """
+        get a random scan
+        """
+        if fdr:
+            sql = "select sp.Spectrum, p.Sequence, p.PeptideID, p.SpectrumID from spectrumheaders sh left join spectra sp on (sp.UniqueSpectrumID=sh.UniqueSpectrumID) left join peptides p on (sh.SpectrumID=p.SpectrumID) WHERE p.ConfidenceLevel >= %d and p.SearchEngineRank <= %d" % (self.clvl, self.srank)
             try:
-                mods = self.mods[int(pepId)]
-                # print mods
-                for modId, modPosition in zip(mods[0].split(','),mods[1].split(',')):
-                    modEntry = self.modTable[int(modId)]
-                    scanObj.addModification(sequence[int(modPosition)], modPosition, modEntry[1], modEntry[0])
-            except KeyError:
-                pass
-            scanObj.peptide = sequence
-            scanObj.rank = searchRank
-            scanObj.confidence = confidence
-            scanObj.acc = i[4]
-            scanObj.charge = i[6]
-            scanObj.rt = i[7]
-            fName = self.sFileMap[i[10]]
-            fScan = i[8]
-            lScan = i[9]
-            sid = '%s.%s.%s'%(fName, fScan,lScan)
-            scanObj.title = sid
-            scanObj.id = sid
-            scanObj.spectrumId=i[5]
-            objs.append(scanObj)
-        return objs
+                self.cur.execute(sql)
+            except sqlite3.OperationalError:
+                sql = "select sp.Spectrum, p.Sequence, p.PeptideID, p.SpectrumID from spectrumheaders sh left join spectra sp on (sp.UniqueSpectrumID=sh.UniqueSpectrumID) left join peptides p on (sh.SpectrumID=p.SpectrumID) WHERE p.ConfidenceLevel >= %d" % self.clvl
+                self.cur.execute(sql)
+        else:
+            sql = "select sp.Spectrum, p.Sequence, p.PeptideID, p.SpectrumID from spectrumheaders sh left join spectra sp on (sp.UniqueSpectrumID=sh.UniqueSpectrumID) left join peptides p on (sh.SpectrumID=p.SpectrumID)"
+            self.cur.execute(sql)
+        while True:
+            results = self.cur.fetchmany(1000)
+            if not results:
+                break
+            for tup in results:
+                scan = self.parseFullScan(tup, modifications=modifications)
+                scan.spectrumId = tup[3]
+                yield scan
+        yield None
 
-    def parseFullScan(self, i):
-        """
-        parses scan info for giving a Spectrum Obj for plotting. takes significantly longer since it has to unzip/parse xml
-        """
-        scanObj = PeptideObject()
-        sInfo = i[0]
+    def decompressScanInfo(self, scanObj, zip):
+        sInfo = zip
         fp = StringIO.StringIO(sInfo)
         zf = zipfile.ZipFile(fp, 'r')
-        peptide = str(i[1])
-        pid=i[2]
-        sql = 'select aam.ModificationName,pam.Position,aam.DeltaMass from peptidesaminoacidmodifications pam left join aminoacidmodifications aam on (aam.AminoAcidModificationID=pam.AminoAcidModificationID) where pam.PeptideID=%s'%pid
-        for row in self.conn.execute(sql):
-            scanObj.addModification(peptide[row[1]], str(row[1]), str(row[2]), row[0])
-        scanObj.peptide = peptide
         for j in zf.namelist():
             msInfo = zf.read(j)
             msStr = msInfo.split('\n')
@@ -1015,9 +1181,11 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
                         scanIndex = finfo.index(' ScanNumber=')+1
                         sid = '%s.%s.%s'%(fName, finfo[scanIndex],finfo[scanIndex])
                         #sid = '%s.%s.%s'%(fName, finfo[2],finfo[2])
+                        scanObj.file = fName
                         scanObj.title = sid
                         scanObj.id = sid
                         scanObj.rt = float(finfo[7])
+                        scanObj.rawId = finfo[scanIndex]
                         master_scan = finfo[5]
                         stage=1
                 elif stage == 1:
@@ -1058,9 +1226,60 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
                 master_scanobj.scans += ms1_scans
                 master_scanobj.scans.sort(key=lambda x: x[0])
                 scanObj.ms1_scan = master_scanobj
+            return True
+        return False
+
+    def parseScan(self, i, modifications=True, full=False):
+#sql = 'select GROUP_CONCAT(p.ConfidenceLevel),GROUP_CONCAT(p.SearchEngineRank),GROUP_CONCAT(p.Sequence),GROUP_CONCAT(p.PeptideID), GROUP_CONCAT(pp.ProteinID), p.SpectrumID, sh.Charge, sh.RetentionTime, sh.FirstScan, sh.LastScan, mp.FileID from peptides p join peptidesproteins pp on (p.PeptideID=pp.PeptideID) left join spectrumheaders sh on (sh.SpectrumID=p.SpectrumID) left join masspeaks mp on (sh.MassPeakID=mp.MassPeakID) where p.PeptideID IS NOT NULL and p.ConfidenceLevel = 1 and p.SearchEngineRank = 1 GROUP BY p.SpectrumID'
+        objs = []
+        self.index+=1
+        added = set([])#for some reason redundant scans appear
+        for confidence, searchRank, sequence, pepId in zip(i[0].split(','),i[1].split(','),i[2].split(','),i[3].split(',')):
+            if (sequence,pepId) in added:
+                continue
+            else:
+                added.add((sequence,pepId))
+            scanObj = PeptideObject()
+            try:
+                mods = self.mods[int(pepId)]
+                # print mods
+                for modId, modPosition in zip(mods[0].split(','),mods[1].split(',')):
+                    modEntry = self.modTable[int(modId)]
+                    scanObj.addModification(sequence[int(modPosition)], modPosition, modEntry[1], modEntry[0])
+            except KeyError:
+                pass
+            scanObj.peptide = sequence
+            scanObj.rank = searchRank
+            scanObj.confidence = confidence
+            scanObj.acc = i[4]
+            scanObj.charge = i[6]
+            scanObj.rt = i[7]
+            fName = self.sFileMap[i[10]]
+            fScan = i[8]
+            lScan = i[9]
+            sid = '%s.%s.%s'%(fName, fScan,lScan)
+            scanObj.title = sid
+            scanObj.id = sid
+            scanObj.spectrumId=i[5]
+            scanObj.rawId = lScan
+            objs.append(scanObj)
+        return objs
+
+    def parseFullScan(self, i, modifications=True):
+        """
+        parses scan info for giving a Spectrum Obj for plotting. takes significantly longer since it has to unzip/parse xml
+        """
+        scanObj = PeptideObject()
+        peptide = str(i[1])
+        pid=i[2]
+        if modifications:
+            sql = 'select aam.ModificationName,pam.Position,aam.DeltaMass from peptidesaminoacidmodifications pam left join aminoacidmodifications aam on (aam.AminoAcidModificationID=pam.AminoAcidModificationID) where pam.PeptideID=%s'%pid
+            for row in self.conn.execute(sql):
+                scanObj.addModification(peptide[row[1]], str(row[1]), str(row[2]), row[0])
+        scanObj.peptide = peptide
+        if self.decompressScanInfo(scanObj, i[0]):
             return scanObj
-        else:
-            return None
+        return None
 
     def next(self):
         if not self.scans:
@@ -1073,7 +1292,7 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
                     self.scans = self.master_scans
                 else:
                     raise StopIteration
-            self.scans = self.parseScan(i)
+            self.scans = self.parseScan(i, full=self.full)
             if not self.scans:
                 if self.master_scans:
                     if isinstance(self.master_scans, dict):
@@ -1088,4 +1307,10 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
         return scan
 
     def getProgress(self):
-        return self.index*100/self.nrows
\ No newline at end of file
+        return self.index*100/self.nrows
+
+    def getPeptide(self, peptide=None):
+        sql = "select sp.Spectrum, p.Sequence, p.PeptideID from spectrumheaders sh left join spectra sp on (sp.UniqueSpectrumID=sh.UniqueSpectrumID) left join peptides p on (sh.SpectrumID=p.SpectrumID) where p.Sequence = '%s'"%(peptide)
+        self.cur.execute(sql)
+        for i in self.cur.fetchall():
+            yield self.parseFullScan(i)
\ No newline at end of file
