@@ -16,15 +16,16 @@ parser = CustomParser(description = description)
 parser.add_fasta()
 parser.add_argument('-t', '--type', help="The type of fasta file (default protein).", choices=['prot','nt'], type=str, default='prot')
 parser.add_argument('--frame', help="If using a nucleotide file, translate in how many frames?", choices=[1,3,6], type=int)
-parser.add_argument('--genome', help="Are we translating a genome? This will keep chromosome positions in the header.", action='store_true', default=False)
+parser.add_argument('--genome', help="Are we translating a genome? This will keep chromosome positions in the header.", action='store_true')
 parser.add_out()
 parser.add_enzyme()
-parser.add_argument('--unique', help="Only return unique peptides per cleavage", action='store_true', default=False)
+parser.add_argument('--unique', help="Only return unique peptides per cleavage", action='store_true')
 
 def main():
     args = parser.parse_args()
     file_name = args.fasta
     enzyme_choice = args.enzyme
+    enzyme_pattern = args.enzyme_pattern
     digest_type = args.type
     digest_frame = args.frame
     digest_negative = False
@@ -47,7 +48,10 @@ def main():
         sys.stderr.write("Nucleotide digestions must specify the frame.\n")
         return 1
     fasta_file = fasta.FastaIterator(file_name)
-    enzyme = digest.Enzyme( enzyme=enzyme_choice )
+    if enzyme_pattern:
+        enzyme = digest.Enzyme(pattern=enzyme_pattern)
+    elif enzyme_choice:
+        enzyme = digest.Enzyme(enzyme=enzyme_choice)
     with args.out as o:
         if digest_type == 'nt':
             for header, sequence in fasta_file:
